from terrain import Terrain

from player import Player

from euclid import Point3, Plane, Vector3
ORIGIN = Point3(0,0,0) # also in constants.py with float coords
from math import pi, sin, cos

import pyglet
from pyglet.window import key

def clamp(x, x1, x2): # warning: differs from the other def of this
    # note: if range is inconsistent, x1 (minumum allowed)
    # takes precedence over x2 (max allowed)
    return max(x1, min(x2, x))

class World:
    keyboard = None # should be set externally (kluge) to a KeyStateHandler
    _players = ()
    
    def __init__(self, N, batch, group = None):
        self.terrain = Terrain( ORIGIN, Point3(1,0,0), Point3(0,0,1), N, N, noise = 0.1)
        self.terrain.add_to_batch(batch, group)

        self.player = Player()
        self.player.add_to_batch(batch, group)
        self._players = [self.player]
        self._bg = batch, group
##        if 1:
##            # just to see if it works (controls might affect both of them for now!
##            # some do, some don't, depending on implem -- e.g. motion does, jump doesn't, key.E toggle doesn't.)
##            # (and the players have no interaction for now)
##            self.player2 = Player(Point3(20,6,20))
##            self.player2.add_to_batch(batch, group)

    def players(self):
        return list(self._players)
    
    def update(self, dt):
        if dt == 0.0:
            ## print "dt is 0!"
            # this happens once when we start;
            # intercept it to prevent division by 0
            return

        for player in self.players():

            what_to_do = player.update(dt, self.terrain, self.keyboard) or {}

            make_hill = what_to_do.get('make_hill', False)
            make_depression = what_to_do.get('make_depression', False)
            recolor = what_to_do.get('recolor', False)
            add_player = what_to_do.get('add_player', False) # false or its initial position
            
            if recolor:
                # for debugging, or for fun, recolor the under-player quad
                self.terrain.recolor_vertices_near(player.pos, 0) # radius = 0: only one quad
            
            if make_hill:
                def heightfunc(old, dx, dz): # note: uses old only for clamp limits
                    goal = max(0., (9. - (dx*dx+dz*dz))) * 0.3
                    rate = dt * 2. * goal/3.
                    return clamp( old + rate, old, goal )
                self.terrain.raise_vertices_near( player.pos, 5., heightfunc)

            if make_depression:
                # bugs: not symmetric with make_hill (maybe the 0.?); doesn't undo it either (need rate proportional to goal?)
                def heightfunc(old, dx, dz):
                    goal = max(0., (9. - (dx*dx+dz*dz))) * 0.3
                    rate = dt * 2. * goal/3.
                    return - clamp( - old + rate, - old, goal )
                self.terrain.raise_vertices_near( player.pos, 5., heightfunc)

            if add_player:
                new = Player(add_player, other = True)
                batch, group = self._bg
                new.add_to_batch(batch, group)
                self._players.append(new)
            

    def start(self): ### XXX probably this belongs elsewhere 
        pyglet.clock.schedule(self.update)

    pass
